@@ -1,8 +1,8 @@
-# copyright 2015-2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free
+# the terms of the GNU Lesser General Public License as publishged by the Free
 # Software Foundation, either version 2.1 of the License, or (at your option)
 # any later version.
 #
@@ -16,7 +16,6 @@
 """cubicweb-saem-ref test for OAI-PMH export"""
 
 from collections import namedtuple
-from datetime import datetime, timedelta
 from functools import wraps
 import time
 
@@ -24,16 +23,17 @@ from dateutil.parser import parse as parse_date
 from lxml import etree
 import pytz
 
-from logilab.common import tempattr
-
+from cubicweb import __pkginfo__
 from cubicweb.devtools.testlib import CubicWebTC
+from cubicweb.pyramid.test import PyramidCWTest
 
-from cubes.oaipmh import isodate
-from cubes.oaipmh.entities import OAIComponent
 from cubes.oaipmh.views import OAIError, OAIRequest
 
+from cubicweb_saem_ref import permanent_url
+
 import testutils
 
+CW_324 = __pkginfo__.numversion < (3, 25)
 
 EntityInfo = namedtuple('EntityInfo', ['ark', 'url', 'eid'])
 
@@ -64,23 +64,18 @@ class OAIComponentTC(CubicWebTC, OAITestMixin):
             oai = self.oai_component(cnx)
             setspecs = oai.__setspecs__
             self.assertCountEqual(setspecs.keys(),
-                                  ['agent', 'organizationunit',
+                                  ['agent', 'organizationunit', 'organization',
+                                   'authorityrecord',
                                    'conceptscheme', 'profile', 'concept'])
             self.assertCountEqual(setspecs['organizationunit'].keys(), ['role'])
             self.assertCountEqual(setspecs['profile'].keys(),
                                   ['transferring_agent'])
             self.assertEqual(setspecs['conceptscheme'].keys(), [])
-
-    def test_match_setspec(self):
-        with self.admin_access.repo_cnx() as cnx:
-            oai = self.oai_component(cnx)
-            with self.assertRaises(ValueError) as cm:
-                oai.match_setspec(u'cwuser')
-            self.assertIn('no setspec matching cwuser', str(cm.exception))
+            self.assertEqual(setspecs['authorityrecord'].keys(), ['used_by'])
 
     def test_setspecs(self):
         with self.admin_access.repo_cnx() as cnx:
-            testutils.organization_unit(
+            arch = testutils.organization_unit(
                 cnx, u'arch', archival_roles=[u'archival'])
             ark = testutils.organization_unit(
                 cnx, u'dep', archival_roles=[u'deposit']).ark
@@ -89,6 +84,10 @@ class OAIComponentTC(CubicWebTC, OAITestMixin):
             publish_entity(scheme)
             expected = (
                 'agent',
+                'authorityrecord',
+                'authorityrecord:used_by:%s' % arch.ark,
+                'authorityrecord:used_by:%s' % ark,
+                'organization',
                 'organizationunit',
                 'organizationunit:role:producer',
                 'organizationunit:role:deposit',
@@ -105,7 +104,7 @@ class OAIComponentTC(CubicWebTC, OAITestMixin):
             self.assertCountEqual(setspecs, expected)
 
 
-def setup_agents(cnx):
+def setup_organization_units(cnx):
     """Create three agents (alice, bobby and draft), publish alice and bobby but not draft, and return
     the (ark, url, eid) attributes of alice and bobby.
     """
@@ -120,17 +119,17 @@ def setup_agents(cnx):
     testutils.organization_unit(cnx, u'draft', archival_roles=[u'enquirer'])
     cnx.commit()
     return (
-        EntityInfo(alice.ark, alice.absolute_url(), alice.eid),
-        EntityInfo(bobby.ark, bobby.absolute_url(), bobby.eid),
+        EntityInfo(alice.ark, permanent_url(alice), alice.eid),
+        EntityInfo(bobby.ark, permanent_url(bobby), bobby.eid),
     )
 
 
 class OAIRequestTC(CubicWebTC):
 
-    def test_rset_from_setspec(self):
-        """Test `rset_from_setspec` function for exceptions."""
+    def test_rset(self):
+        """Test `rset` function for exceptions."""
         with self.admin_access.repo_cnx() as cnx:
-            return setup_agents(cnx)
+            return setup_organization_units(cnx)
         with self.admin_access.web_request() as req:
             for setspec, msg in [
                 ('organizationunit:zen', 'invalid set specifier organizationunit:zen'),
@@ -146,30 +145,6 @@ class OAIRequestTC(CubicWebTC):
                         oairq.rset_from_setspec(req)
                     self.assertEqual(cm.exception.errors, {'noRecordsMatch': msg})
 
-    def test_rset_from_setspec_token(self):
-        with self.admin_access.repo_cnx() as cnx:
-            alice, bobby = setup_agents(cnx)
-
-        def oairq(**kwargs):
-            return OAIRequest('http://testing.fr', setspec='organizationunit', **kwargs)
-
-        with self.admin_access.web_request() as req:
-            with tempattr(OAIComponent, 'max_result_size', 1):
-                rset, token = oairq().rset_from_setspec(req)
-                self.assertEqual(len(rset), 1)
-                self.assertEqual(token, str(bobby.eid))
-                rset, token = oairq(resumption_token=str(bobby.eid)).rset_from_setspec(req)
-                self.assertEqual(len(rset), 1)
-                self.assertIsNone(token)
-            # All results are returned, the resumptionToken is None.
-            rset, token = oairq().rset_from_setspec(req)
-            self.assertEqual(len(rset), 2)
-            self.assertIsNone(token)
-            # and this, even if the user specified an input token.
-            rset, token = oairq(resumption_token='1').rset_from_setspec(req)
-            self.assertEqual(len(rset), 2)
-            self.assertIsNone(token)
-
 
 def no_validate_xml(method):
     """Disable XML schema validation, often because the underlying metadata
@@ -184,21 +159,39 @@ def no_validate_xml(method):
     return wrapper
 
 
-class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
+class OAIPMHViewsTC(PyramidCWTest, OAITestMixin, testutils.XmlTestMixin):
+    settings = {
+        # to get clean traceback in tests (instead of in an html error page)
+        'cubicweb.bwcompat': False,
+        # avoid noise in test output (UserWarning: !! WARNING WARNING !! you
+        # should stop this instance)
+        'cubicweb.session.secret': 'x',
+        'cubicweb.auth.authtkt.session.secret': 'x',
+        'cubicweb.auth.authtkt.persistent.secret': 'x',
+    }
 
     _validate_xml = True
     _debug_xml = False
 
-    def oai_request(self, req, **formparams):
-        req.form.update(formparams)
-        out = self.app_handle_request(req, 'oai')
-        if self._validate_xml:
-            self.assertXmlValid(out, self.datapath('OAI-PMH.xsd'), debug=self._debug_xml)
-        return out
+    if CW_324:
+        # webapp is initialized with https as url scheme, which trigger usage of https-url instead
+        # of base-url in 3.24, but tests below rely in base-url being used.
+        @classmethod
+        def init_config(cls, config):
+            super(PyramidCWTest, cls).init_config(config)
+            config.global_set_option('https-url', config['base-url'])
 
-    def _setup_agents(self, **kwargs):
+    def oai_request(self, req, **formparams):
+        response = self.webapp.get('/oai', formparams)
+        self.assertEqual(response.headers['Content-Type'], 'text/xml; charset=UTF-8')
+        if self._validate_xml:
+            self.assertXmlValid(response.body, self.datapath('OAI-PMH.xsd'),
+                                debug=self._debug_xml)
+        return response.body
+
+    def _setup_organization_units(self, **kwargs):
         with self.admin_access.repo_cnx() as cnx:
-            return setup_agents(cnx, **kwargs)
+            return setup_organization_units(cnx, **kwargs)
 
     def test_xml_attributes_and_namespaces(self):
         """Check XML attributes and namespace declaration of the response."""
@@ -217,27 +210,6 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
             }
             self.assertEqual(xml.attrib, attrib)
 
-    def test_responsedate(self):
-        """Check <responseDate>"""
-        with self.admin_access.web_request() as req:
-            # Simple request, generating an error but enough to get a properly
-            # formatter response.
-            now = datetime.utcnow()
-            result = self.oai_request(req)
-        # Check tzinfo abbreviation is present (Z for UTC).
-        self.assertIn('<responseDate>{0}</responseDate>'.format(isodate(now)),
-                      result)
-
-    def _setup_agents_deleted(self):
-        alice, bobby = self._setup_agents()
-        with self.admin_access.repo_cnx() as cnx:
-            entity = cnx.entity_from_eid(bobby.eid)
-            # ensure transition date is greater by at least 1s
-            time.sleep(1)
-            date_before = entity.modification_date.replace(tzinfo=pytz.utc)
-            deprecate_entity(entity)
-        return alice, bobby, date_before
-
     def _setup_schemes(self):
         with self.admin_access.repo_cnx() as cnx:
             pscheme = testutils.setup_scheme(cnx, u'public',
@@ -287,96 +259,55 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
                       result)
         self.assertNotIn(dscheme.ark, result)
 
-    def test_listidentifiers_noset(self):
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers')
-        self.assertIn('<request>http://testing.fr/cubicweb/oai</request>',
-                      result)
-        self.assertIn(('<error code="badArgument">'
-                       'ListIdentifiers verb requires "set" restriction'
-                       '</error>'),
-                      result)
-
     def _setup_profile(self, **kwargs):
-        """Create a "published" SEDAProfile and return its identifier"""
+        """Create a "published" SEDAArchiveTransfer and return it."""
         with self.admin_access.repo_cnx() as cnx:
-            profile = testutils.setup_seda_profile(cnx, **kwargs)
+            profile = testutils.setup_profile(cnx, **kwargs)
             cnx.commit()
             publish_entity(profile)
-            return profile.ark
-
-    def test_resumptiontoken_listidentifiers(self):
-        alice, bobby = self._setup_agents()
-
-        def check_request(expected_arks, token=None):
-            with self.admin_access.web_request() as req:
-                result = self.oai_request(
-                    req, verb='ListIdentifiers', set='organizationunit',
-                    resumptionToken=token)
-                # Ensure there are as many <identifier> tag than expected items.
-                self.assertEqual(result.count('<identifier>'), len(expected_arks))
-                for ark in expected_arks:
-                    self.assertIn(
-                        '<identifier>ark:/{0}</identifier>'.format(ark), result)
-                return result
-
-        # less results than pagination limit
-        result = check_request([alice.ark, bobby.ark])
-        # All results returned, no resumptionToken is request -> no
-        # resumptionToken in response.
-        self.assertNotIn('resumptionToken', result)
-        with tempattr(OAIComponent, 'max_result_size', 1):
-            # max_result_size = 1, 1st batch
-            result = check_request([alice.ark])
-            # token + expirationDate
-            root = etree.fromstring(result)
-            token = root.find('.//{%s}resumptionToken' % root.nsmap[None])
-            date = parse_date(token.attrib['expirationDate'])
-            self.assertLess(datetime.now(pytz.utc) + timedelta(hours=1) - date,
-                            timedelta(seconds=1))
-            self.assertEqual(token.text, str(bobby.eid))
-            # max_result_size = 1, 2nd batch
-            result = check_request([bobby.ark], token=str(bobby.eid))
-            # max_result_size = 1, no more result
-            # No more results -> empty resumptionToken.
-            self.assertIn('<resumptionToken/>', result)
-            # Empty result, probably a badResumptionToken.
-            result = check_request([], token=str(bobby.eid + 1))
-            self.assertIn('badResumptionToken', result)
+            profile.ark  # prefetch attribute for later usage
+            return profile
 
     def test_listidentifiers(self):
-        alice, bobby = [x.ark for x in self._setup_agents()]
-        profile_id = self._setup_profile()
+        alice, bobby = [x.ark for x in self._setup_organization_units()]
+        profile = self._setup_profile()
         with self.admin_access.web_request() as req:
             result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='organizationunit')
+                                      set='organizationunit',
+                                      metadataPrefix='rdf')
             self.assertIn('<identifier>ark:/{0}</identifier>'.format(bobby), result)
             self.assertIn('<identifier>ark:/{0}</identifier>'.format(alice), result)
             draft = req.execute('Any A WHERE X ark A, X name "draft"')[0][0]
             self.assertNotIn(draft, result)
             result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='organizationunit:role:producer')
+                                      set='organizationunit:role:producer',
+                                      metadataPrefix='rdf')
             self.assertIn('<identifier>ark:/{0}</identifier>'.format(alice),
                           result)
             self.assertNotIn('<identifier>ark:/{0}</identifier>'.format(bobby),
                              result)
             result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='organizationunit:role:producer')
+                                      set='organizationunit:role:producer',
+                                      metadataPrefix='rdf')
             self.assertIn('<identifier>ark:/{0}</identifier>'.format(alice), result)
             self.assertNotIn('<identifier>ark:/{0}</identifier>'.format(bobby), result)
-            result = self.oai_request(req, verb='ListIdentifiers', set='conceptscheme')
-            result = self.oai_request(req, verb='ListIdentifiers', set='profile')
-            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile_id), result)
+            result = self.oai_request(req, verb='ListIdentifiers',
+                                      set='conceptscheme',
+                                      metadataPrefix='rdf')
+            result = self.oai_request(req, verb='ListIdentifiers',
+                                      set='profile', metadataPrefix='seda02xsd')
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile.ark), result)
 
     def test_listidentifiers_setspec_wfstate(self):
         """Check workflow state condition is statisfied with a multi-level
         setspec `organizationunit:role:enquirer`.
         """
-        alice, bobby = [x.ark for x in self._setup_agents()]
+        alice, bobby = [x.ark for x in self._setup_organization_units()]
         self._setup_profile()
         with self.admin_access.web_request() as req:
             result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='organizationunit:role:enquirer')
+                                      set='organizationunit:role:enquirer',
+                                      metadataPrefix='rdf')
             self.assertIn('<identifier>ark:/{0}</identifier>'.format(bobby), result)
             draft = req.execute('Any A WHERE X ark A, X name "draft"')[0][0]
             self.assertNotIn(draft, result)
@@ -385,10 +316,18 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
         """Check <header> element (status attribute, datestamp) for deleted
         records in ListIdentifiers response.
         """
-        alice, bobby, date_before = self._setup_agents_deleted()
+        alice, bobby = self._setup_organization_units()
+        with self.admin_access.repo_cnx() as cnx:
+            entity = cnx.entity_from_eid(bobby.eid)
+            # ensure transition date is greater by at least 1s
+            time.sleep(1)
+            date_before = entity.modification_date.replace(tzinfo=pytz.utc)
+            deprecate_entity(entity)
+
         with self.admin_access.web_request() as req:
             result = self.oai_request(req, verb='ListIdentifiers',
-                                      set='organizationunit')
+                                      set='organizationunit',
+                                      metadataPrefix='rdf')
             self.assertIn(
                 '<identifier>ark:/{0}</identifier>'.format(bobby.ark), result)
             self.assertIn('<header status="deleted">', result)
@@ -400,22 +339,12 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
             date = parse_date(datestamp)
             self.assertGreater(date, date_before.replace(microsecond=0))
 
-    def test_listrecords_noset(self):
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListRecords')
-        self.assertIn('<request>http://testing.fr/cubicweb/oai</request>',
-                      result)
-        self.assertIn(('<error code="badArgument">'
-                       'ListRecords verb requires "set" restriction'
-                       '</error>'),
-                      result)
-
     @no_validate_xml
     def test_listrecords(self):
-        alice, bobby = self._setup_agents()
+        alice, bobby = self._setup_organization_units()
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListRecords',
-                                      set='organizationunit')
+            result = self.oai_request(req, verb='ListRecords', set='organizationunit',
+                                      metadataPrefix='rdf')
             self.assertIn(
                 '<rdf:Description rdf:about="{0}">'.format(bobby.url), result)
             self.assertIn(
@@ -423,71 +352,66 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
             draft = req.execute('Any A WHERE X ark A, X name "draft"')[0][0]
             self.assertNotIn(draft, result)
             result = self.oai_request(
-                req, verb='ListRecords', set='organizationunit:role:producer')
+                req, verb='ListRecords', set='organizationunit:role:producer',
+                metadataPrefix='rdf')
             self.assertIn(
                 '<rdf:Description rdf:about="{0}">'.format(alice.url), result)
             self.assertNotIn(bobby.url, result)
 
-    @no_validate_xml
-    def test_listrecord_deleted(self):
-        """Check absence of <metadata> or <about> elements for deleted
-        records in ListRecords response.
-        """
-        alice, bobby, _ = self._setup_agents_deleted()
-        with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListRecords', set='organizationunit')
-            # no metadata for bobby
-            self.assertNotIn(
-                '<rdf:Description rdf:about="{0}">'.format(bobby.url), result)
-            # but record present
-            self.assertIn('<identifier>ark:/{0}</identifier>'.format(bobby.ark), result)
-            self.assertIn(
-                '<rdf:Description rdf:about="{0}">'.format(alice.url), result)
-
     @no_validate_xml
     def test_listrecords_profile(self):
         self._setup_profile()
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListRecords', set='profile')
+            result = self.oai_request(req, verb='ListRecords', set='profile',
+                                      metadataPrefix='seda02xsd')
         # Just ensure a SEDA profile (xsd) is present in response.
         self.assertIn('<metadata><xsd:schema', result)
         self.assertIn('fr:gouv:ae:archive:draft:standard_echange_v0.2', result)
 
     @no_validate_xml
     def test_listidentifiers_profile_setspec(self):
-        """Check setspec restriction for SEDAProfile."""
-        profile_id = self._setup_profile()
+        """Check setspec restriction for SEDAArchiveTransfer."""
+        profile = self._setup_profile()
         with self.admin_access.cnx() as cnx:
             tagent_eid = testutils.organization_unit(
                 cnx, u'blah', archival_roles=[u'deposit']).eid
             cnx.commit()
-        profile2_id = self._setup_profile(seda_transferring_agent=tagent_eid)
+        profile2 = self._setup_profile()
+        with self.admin_access.cnx() as cnx:
+            cnx.entity_from_eid(tagent_eid).cw_set(use_profile=profile2.eid)
+            cnx.commit()
         with self.admin_access.web_request() as req:
-            transferring_agent_ark = req.execute(
-                'Any K WHERE P ark "%s", P seda_transferring_agent A, A ark K'
-                % profile_id)[0][0]
+            transferring_agent_ark = req.entity_from_eid(tagent_eid).ark
             result = self.oai_request(
                 req, verb='ListIdentifiers',
-                set='profile:transferring_agent:' + transferring_agent_ark)
-            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile_id), result)
-            self.assertNotIn(profile2_id, result)
+                set='profile:transferring_agent:' + transferring_agent_ark,
+                metadataPrefix='seda02xsd')
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile2.ark), result)
+            self.assertNotIn(profile.ark, result)
         # Check that without a setspec, we get all profiles.
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='ListIdentifiers', set='profile')
-            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile_id), result)
-            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile2_id), result)
+            result = self.oai_request(req, verb='ListIdentifiers',
+                                      set='profile', metadataPrefix='seda02xsd')
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile.ark), result)
+            self.assertIn('<identifier>ark:/{0}</identifier>'.format(profile2.ark), result)
 
     @no_validate_xml
     def test_getrecord_profile(self):
-        profile_id = self._setup_profile()
+        profile = self._setup_profile()
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='GetRecord')
-            self.assertIn('<error code="badArgument">', result)
-            result = self.oai_request(req, verb='GetRecord', identifier='999')
-            self.assertIn('<error code="idDoesNotExist"', result)
-            result = self.oai_request(req, verb='GetRecord', identifier='ark:/' + profile_id)
-            self.assertIn('<metadata><xsd:schema', result)
-            self.assertIn('fr:gouv:ae:archive:draft:standard_echange_v0.2', result)
+            with self.subTest(md_prefix='seda02xsd'):
+                result = self.oai_request(req, verb='GetRecord',
+                                          identifier='ark:/' + profile.ark,
+                                          metadataPrefix='seda02xsd')
+                self.assertIn('<metadata><xsd:schema', result)
+                self.assertIn('http://www.w3.org/2001/XMLSchema', result)
+            for md_prefix in ['seda02rng', 'seda1rng', 'seda2rng']:
+                with self.subTest(md_prefix=md_prefix):
+                    result = self.oai_request(req, verb='GetRecord',
+                                              identifier='ark:/' + profile.ark,
+                                              metadataPrefix=md_prefix)
+                    self.assertIn('<metadata><rng:grammar', result)
+                    self.assertIn('http://www.w3.org/2001/XMLSchema', result)
 
     @no_validate_xml
     def test_getrecord_agent(self):
@@ -498,11 +422,27 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
             publish_entity(agent)
             ark = agent.ark
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='GetRecord', identifier='ark:/' + ark)
+            result = self.oai_request(req, verb='GetRecord',
+                                      identifier='ark:/' + ark,
+                                      metadataPrefix='rdf')
             self.assertIn('name>toto</', result)
             self.assertIn('<rdf:type rdf:resource="http://xmlns.com/foaf/0.1/Person"/>',
                           result)
 
+    @no_validate_xml
+    def test_getrecord_organization(self):
+        with self.admin_access.repo_cnx() as cnx:
+            org = testutils.authority_with_naa(cnx)
+            cnx.commit()
+            ark = org.ark
+        with self.admin_access.web_request() as req:
+            result = self.oai_request(req, verb='GetRecord',
+                                      identifier='ark:/' + ark,
+                                      metadataPrefix='rdf')
+            self.assertIn('title>Default authority</', result)
+            self.assertIn('<rdf:type rdf:resource="http://www.w3.org/ns/org#Organization"/>',
+                          result)
+
     @no_validate_xml
     def test_getrecord_organizationunit(self):
         self._debug_xml = True
@@ -512,29 +452,58 @@ class OAIPMHViewsTC(CubicWebTC, OAITestMixin, testutils.XmlTestMixin):
             publish_entity(orgu)
             ark = orgu.ark
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='GetRecord', identifier='ark:/' + ark)
-            self.assertIn('name>toto</', result)
+            result = self.oai_request(req, verb='GetRecord',
+                                      identifier='ark:/' + ark,
+                                      metadataPrefix='rdf')
+            self.assertIn('title>toto</', result)
             self.assertIn('<rdf:type rdf:resource="http://www.w3.org/ns/org#OrganizationUnit"/>',
                           result)
 
-    def test_getrecord_deleted(self):
-        """Check absence of <metadata> or <about> elements for deleted
-        records in GetRecord response.
-        """
-        alice, bobby, _ = self._setup_agents_deleted()
+    @no_validate_xml
+    def test_getrecord_authority_record(self):
+        with self.admin_access.repo_cnx() as cnx:
+            arecord = testutils.authority_record(cnx, u'M. Person')
+            cnx.commit()
+            arecord.cw_adapt_to('IWorkflowable').fire_transition('publish')
+            cnx.commit()
+            arecord_ark = arecord.ark
         with self.admin_access.web_request() as req:
-            result = self.oai_request(req, verb='GetRecord', identifier='ark:/' + bobby.ark)
-            # record is present
-            self.assertIn('<identifier>ark:/{0}</identifier>'.format(bobby.ark), result)
-            # but not data
-            self.assertNotIn('metadata', result)
-            self.assertNotIn('rdf', result)
+            result = self.oai_request(req, verb='GetRecord',
+                                      identifier='ark:/' + arecord_ark,
+                                      metadataPrefix='eac')
+            self.assertIn('<metadata><eac-cpf', result)
+            self.assertIn('urn:isbn:1-931666-33-4', result)
+
+    @no_validate_xml
+    def test_authorityrecord_from_organizationunit(self):
+        with self.admin_access.cnx() as cnx:
+            org = testutils.authority_with_naa(cnx)
+            ou_ark = testutils.organization_unit(
+                cnx, u'test', archival_roles=('archival', ),
+                reverse_archival_unit=org, authority=org).ark
+            self.create_user(cnx, u'bob', authority=org)
+            cnx.commit()
+        with self.new_access(u'bob').cnx() as cnx:
+            arecord = testutils.authority_record(cnx, u'test')
+            cnx.commit()
+            assert arecord.reverse_use_authorityrecord
+            arecord.cw_adapt_to('IWorkflowable').fire_transition('publish')
+            cnx.commit()
+            ark = arecord.ark
+        with self.admin_access.web_request() as req:
+            result = self.oai_request(
+                req, verb='ListIdentifiers',
+                set='authorityrecord:used_by:{}'.format(ou_ark),
+                metadataPrefix='eac')
+            self.assertIn('<identifier>ark:/' + ark, result)
 
     def test_getrecord_draft(self):
-        self._setup_agents()
+        self._setup_organization_units()
         with self.admin_access.web_request() as req:
             draft = req.execute('Any A WHERE X ark A, X name "draft"')[0][0]
-            result = self.oai_request(req, verb='GetRecord', identifier='ark:/' + draft)
+            result = self.oai_request(req, verb='GetRecord',
+                                      identifier='ark:/' + draft,
+                                      metadataPrefix='rdf')
             self.assertIn('idDoesNotExist', result)
 
 
