@@ -1,5 +1,5 @@
 # coding: utf-8
-# copyright 2015-2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -25,19 +25,25 @@ try:
 except ImportError:
     import unittest
 
+from six import text_type
+
 from yams.schema import role_name
 
-from cubicweb import Binary
 from cubicweb.devtools.testlib import CubicWebTC
-from cubicweb.web.views.actions import CopyAction
 
 from cubes.skos.rdfio import default_graph
 
-from cubes.saem_ref.views.widgets import process_incomplete_date
+from cubicweb_saem_ref.views.widgets import process_incomplete_date
 
 import testutils
 
 
+def eid_from_ark(ark):
+    name = ark.rsplit('/', 1)[-1]
+    num = ''.join(c for c in name if c.isdigit())
+    return int(num)
+
+
 class SetDateTC(unittest.TestCase):
 
     def test_set_date_month_beginning(self):
@@ -73,24 +79,6 @@ class SetDateTC(unittest.TestCase):
 
 class FuncViewsTC(CubicWebTC):
 
-    def test_xmlwrap_component(self):
-        with self.admin_access.cnx() as cnx:
-            bob = testutils.authority_record(cnx, u'bob')
-            uri = cnx.create_entity('ExternalUri', uri=u'http://logilab.fr')
-            cnx.create_entity('EACResourceRelation',
-                              resource_relation_agent=bob,
-                              resource_relation_resource=uri,
-                              xml_wrap=Binary('<plip>plop</plip>'))
-            cnx.commit()
-        with self.admin_access.web_request() as req:
-            rset = req.find('EACResourceRelation')
-            component = self.vreg['ctxcomponents'].select(
-                'saem_ref.xml_wrap', req, rset=rset)
-            content = []
-            component.render_body(content.append)
-        self.assertIn(u'plop', content[0])
-        self.assertEqual(content[0].count(u'plip'), 2)
-
     def test_citation_view(self):
         with self.admin_access.cnx() as cnx:
             bob = testutils.authority_record(cnx, u'bob')
@@ -112,13 +100,14 @@ class FuncViewsTC(CubicWebTC):
             self.assertEqual(without_link_html, u'<i class="truncate">plop</i>')
 
     def test_eac_import_user_has_no_naa(self):
-        regid = 'saem_ref.eac-import'
+        regid = 'eac.import'
         with self.admin_access.web_request() as req:
             output = self.view(regid, req=req, template=None)
             self.assertIn('be in an organization', output)
             self.assertIn('to access this functionnality', output)
         with self.admin_access.web_request() as req:
-            authority = req.cnx.find('Organization', name=u'Default authority').one()
+            authority = req.cnx.create_entity('Organization', name=u'dummy',
+                                              ark=u'123')
             req.user.cw_set(authority=authority)
             req.cnx.commit()
             output = self.view(regid, req=req, template=None)
@@ -126,7 +115,7 @@ class FuncViewsTC(CubicWebTC):
             self.assertIn('to access this functionnality', output)
 
     def test_eac_import_ok(self):
-        regid = 'saem_ref.eac-import'
+        regid = 'eac.import'
         fname = 'FRAD033_EAC_00001_simplified.xml'
         with self.admin_access.web_request() as req:
             req.user.cw_set(authority=testutils.authority_with_naa(req.cnx))
@@ -134,63 +123,14 @@ class FuncViewsTC(CubicWebTC):
             # simply test the form properly render and is well formed
             self.view(regid, req=req, template=None)
             fields = {'file': (fname, open(self.datapath('EAC/' + fname))),
-                      'naa': unicode(naa.eid)}
+                      'naa': text_type(naa.eid)}
             req.form = self.fake_form(regid, fields)
             # now actually test the import
             req.view(regid)
-            # test AuthorityRecord and Agent have been created, the former with a ARK from NAA
-            # specified in form params, the Agent with a ARK from the user's NAA
-            arecord = req.find('AuthorityRecord', name=u'Gironde. Conseil général').one()
+            # test AuthorityRecord have been created with a ARK from NAA
+            # specified in form params
+            arecord = req.find('AuthorityRecord', has_text=u'Gironde. Conseil général').one()
             self.assertTrue(arecord.ark.startswith('1/'), arecord.ark)
-            agent = req.find('Agent', name=u'Delphine Jamet').one()
-            self.assertTrue(agent.ark.startswith('0/'), agent.ark)
-
-    def test_eac_non_unique_isni(self):
-        regid = 'saem_ref.eac-import'
-        fname = 'FRAD033_EAC_00001_simplified.xml'
-        with self.admin_access.client_cnx() as cnx:
-            # ISNI is the same as the agent in EAC file.
-            testutils.authority_record(cnx, u'bob', isni=u'22330001300016')
-            cnx.commit()
-            rset = cnx.find('AuthorityRecord')
-            self.assertEqual(len(rset), 1)
-        with self.admin_access.web_request() as req:
-            req.user.cw_set(authority=testutils.authority_with_naa(req.cnx))
-            # simply test the form properly render and is well formed
-            self.view(regid, req=req, template=None)
-            fields = {'file': (fname, open(self.datapath('EAC/' + fname))),
-                      'naa': unicode(req.user.naa.eid)}
-            req.form = self.fake_form(regid, fields)
-            # now actually test the import
-            html = req.view(regid)
-            self.assertIn('EAC import failed', html)
-            # Still only one AuthorityRecord.
-            rset = req.find('AuthorityRecord')
-            self.assertEqual(len(rset), 1)
-
-    def test_eac_invalid_xml(self):
-        regid = 'saem_ref.eac-import'
-        fname = 'invalid_xml.xml'
-        with self.admin_access.web_request() as req:
-            req.user.cw_set(authority=testutils.authority_with_naa(req.cnx))
-            fields = {'file': (fname, open(self.datapath('EAC/' + fname))),
-                      'naa': unicode(req.user.naa.eid)}
-            req.form = self.fake_form(regid, fields)
-            # now actually test the import
-            html = req.view(regid)
-            self.assertIn('Invalid XML file', html)
-
-    def test_eac_missing_tag(self):
-        regid = 'saem_ref.eac-import'
-        fname = 'missing_tag.xml'
-        with self.admin_access.web_request() as req:
-            req.user.cw_set(authority=testutils.authority_with_naa(req.cnx))
-            fields = {'file': (fname, open(self.datapath('EAC/' + fname))),
-                      'naa': unicode(req.user.naa.eid)}
-            req.form = self.fake_form(regid, fields)
-            # now actually test the import
-            html = req.view(regid)
-            self.assertIn('Missing tag cpfDescription in XML file', html)
 
     def test_highlight_script_execution(self):
         with self.admin_access.client_cnx() as cnx:
@@ -208,10 +148,11 @@ class FuncViewsTC(CubicWebTC):
             scheme = testutils.setup_scheme(cnx, u'my thesaurus', u'toto', u'tata toto')
             concept = scheme.reverse_in_scheme[0]
             cnx.commit()
+            ark = concept.ark
         url = 'http://testing.fr/cubicweb/view?rql=toto&__fromsearchbox=1&subvid=tsearch'
         html = self.http_publish(url)[0]
         self.assertIn(
-            '<a href="http://testing.fr/cubicweb/%s?highlight=toto" title="">' % concept.eid,
+            '<a href="http://testing.fr/cubicweb/ark:/{}?highlight=toto" title="">'.format(ark),
             html)
 
     def test_skos_negociation(self):
@@ -231,16 +172,16 @@ class FuncViewsTC(CubicWebTC):
                 finally:
                     os.unlink(fobj.name)
 
-    def test_download_filename(self):
+    def test_eac_download_filename(self):
         with self.admin_access.web_request() as req:
             cnx = req.cnx
             agent = testutils.authority_record(cnx, u'jim')
             for ark, expected_filename in (
                 (u"", "EAC_{0}.xml".format(agent.eid)),
-                (u"ZZZ/4242", "EAC_ZZZ_4242.xml".format(agent.eid)),
+                (u"ZZZ/4242", "EAC_ZZZ-4242.xml".format(agent.eid)),
             ):
                 agent.cw_set(ark=ark)
-                view = self.vreg['views'].select('saem_ref.eac', req, agent.as_rset())
+                view = self.vreg['views'].select('eac.export', req, agent.as_rset())
                 view.set_request_content_type()
                 self.assertEqual(
                     view._cw.headers_out.getRawHeaders('content-disposition'),
@@ -262,154 +203,59 @@ class FuncViewsTC(CubicWebTC):
             self.assertIn(entity.view('oneline'), content)
             self.assertIn('__redirectparams=tab%3Dblah&amp', content)
 
-
-class AuthorityRecordFormsTC(CubicWebTC):
-
-    def test_no_copy_action(self):
-        with self.admin_access.cnx() as cnx:
-            testutils.authority_record(cnx, u'toto')
-            cnx.commit()
+    def test_prov_activities(self):
         with self.admin_access.web_request() as req:
-            rset = req.find('AuthorityRecord', name=u'toto')
-            actions = self.pactionsdict(req, rset)
-            self.assertNotIn(CopyAction, actions['moreactions'])
-
-    def test_unrelated_authorityrecord(self):
-        from cubes.saem_ref.views.authorityrecord import unrelated_authorityrecord
-        with self.admin_access.cnx() as cnx:
-            ar1 = testutils.authority_record(cnx, u'1').eid
-            ar2 = testutils.authority_record(cnx, u'2').eid
-            # ExternalUri should not appear in choices.
-            cnx.create_entity('ExternalUri', uri=u'http://no-where')
-            cnx.commit()
-        with self.admin_access.web_request() as req:
-            cls = self.vreg['etypes'].etype_class('HierarchicalRelation')(req)
-            form = self.vreg['forms'].select('edition', req, entity=cls)
-            field = form.field_by_name('hierarchical_parent', 'subject')
-            choices = unrelated_authorityrecord(
-                'hierarchical_parent', form, field)
-            expected = [(u'1', unicode(ar1)), (u'2', unicode(ar2))]
-            self.assertCountEqual(choices, expected)
-        # Now with a __linkto.
-        linkto = 'hierarchical_parent:{0}:subject'.format(ar2)
-        with self.admin_access.web_request(__linkto=linkto) as req:
-            cls = self.vreg['etypes'].etype_class('HierarchicalRelation')(req)
-            form = self.vreg['forms'].select('edition', req, entity=cls)
-            field = form.field_by_name('hierarchical_parent', 'subject')
-            choices = unrelated_authorityrecord(
-                'hierarchical_parent', form, field)
-            expected = [(u'2', unicode(ar2))]
-            self.assertCountEqual(choices, expected)
-            field = form.field_by_name('hierarchical_child', 'subject')
-            choices = unrelated_authorityrecord(
-                'hierarchical_child', form, field)
-            expected = [(u'1', unicode(ar1)), (u'2', unicode(ar2))]
-            self.assertCountEqual(choices, expected)
-
-
-class SEDANavigationTC(CubicWebTC):
-
-    def test_breadcrumbs(self):
-        with self.admin_access.cnx() as cnx:
-            scheme = testutils.seda_scheme(
-                cnx, u'seda_document_type_code', u'preferred', u'QWE')
-            doctypecodevalue = scheme.reverse_in_scheme[0]
-            doctypecode = cnx.create_entity(
-                'SEDADocumentTypeCode',
-                seda_document_type_code_value=doctypecodevalue)
-            doc = cnx.create_entity('ProfileDocument',
-                                    seda_document_type_code=doctypecode)
-            profile = cnx.create_entity('SEDAProfile', ark_naa=testutils.naa(cnx))
-            archobj = cnx.create_entity(
-                'ProfileArchiveObject',
-                seda_name=cnx.create_entity('SEDAName'),
-                seda_parent=profile)
-            cnx.commit()
-        with self.admin_access.web_request() as req:
-            archobj = req.entity_from_eid(archobj.eid)
-            profile = req.entity_from_eid(profile.eid)
-            doc = req.entity_from_eid(doc.eid)
-            # ProfileDocument not related through seda_parent, breadcrumbs
-            # leads to /sedalib.
-            breadcrumbs = doc.cw_adapt_to('IBreadCrumbs').breadcrumbs()
-            expected_breadcrumbs = [
-                (u'http://testing.fr/cubicweb/sedalib', u'SEDA components'),
-                doc,
-            ]
-            self.assertEqual(breadcrumbs, expected_breadcrumbs)
-            # ProfileDocument has seda_parent, breadcrumbs leads to parents
-            # (ProfileArchiveObject, SEDAProfile).
-            doc.cw_set(seda_parent=archobj)
+            record = testutils.authority_record(req, u'My record')
+            req.create_entity('Activity', agent=u'007', generated=record)
             req.cnx.commit()
-            doc.cw_clear_all_caches()
-            breadcrumbs = doc.cw_adapt_to('IBreadCrumbs').breadcrumbs()
-            expected_breadcrumbs = [profile, archobj, doc]
-            self.assertEqual(breadcrumbs, expected_breadcrumbs)
+            table_html = record.view('prov.activity-generated')
+            self.assertIn('admin', table_html)
+            self.assertIn('007', table_html)
 
-
-class SEDAFormsTC(CubicWebTC):
-
-    def _build_seda_scheme(self, rtype_name, label_kind=u'preferred'):
-        with self.admin_access.cnx() as cnx:
-            labels = (u'pif', u'paf', u'pouf')
-            testutils.seda_scheme(cnx, rtype_name, label_kind, *labels)
-            cnx.commit()
-        return labels
-
-    def test_seda_document_type_code_vocabulary(self):
-        """Check that we get correct values in the combo box for document type code."""
-        labels = self._build_seda_scheme(u'seda_document_type_code')
+    def test_related_entities_navigation(self):
         with self.admin_access.web_request() as req:
-            doc = self.vreg['etypes'].etype_class('SEDADocumentTypeCode')(req)
-            form = self.vreg['forms'].select('edition', req, entity=doc)
-            field = form.field_by_name('seda_document_type_code_value', 'subject')
-            form_labels = [label for label, _ in field.vocabulary(form)]
-            self.assertCountEqual(labels, form_labels)
+            ou = testutils.organization_unit(req, u'Archival Dptmt', ['archival'])
+            testutils.authority_record(req.cnx, u'bob', reverse_use_authorityrecord=ou)
+            testutils.authority_record(req.cnx, u'alice', reverse_use_authorityrecord=ou)
+            req.cnx.commit()
 
-    def test_seda_file_type_code_vocabulary(self):
-        """Check that we get correct values in the combo box for file type code."""
-        labels = self._build_seda_scheme(u'seda_file_type_code',
-                                         u'alternative')
-        with self.admin_access.web_request() as req:
-            doc = self.vreg['etypes'].etype_class('SEDAFileTypeCode')(req)
-            form = self.vreg['forms'].select('edition', req, entity=doc)
-            field = form.field_by_name('seda_file_type_code_value', 'subject')
-            values = list(field.vocabulary(form))
-            form_labels = [label for label, _ in values]
-            self.assertCountEqual(form_labels, labels)
+            vid = 'saem.ou.authorityrecords'
+            view = req.vreg['views'].select(vid, req, rset=ou.as_rset())
+            html = view.render()
+            nav_link = view.page_navigation_url(None, None, {})
 
-    def test_seda_character_set_code_vocabulary(self):
-        """Check that we get correct values in the combo box for character set code."""
-        labels = self._build_seda_scheme(u'seda_character_set_code',
-                                         u'alternative')
-        with self.admin_access.web_request() as req:
-            doc = self.vreg['etypes'].etype_class('SEDACharacterSetCode')(req)
-            form = self.vreg['forms'].select('edition', req, entity=doc)
-            field = form.field_by_name('seda_character_set_code_value', 'subject')
-            values = list(field.vocabulary(form))
-            form_labels = [label for label, _ in values]
-            self.assertCountEqual(form_labels, labels)
+            expected_divid = '{}{}'.format(vid.replace('.', '_'), ou.eid)
+            self.assertIn('<div id="{}">'.format(expected_divid), html)
+            self.assertIn('#' + expected_divid, nav_link)
+            params = json.loads('{{ {} }}'.format(nav_link.split('{')[1].split('}')[0]))
+            self.assertEqual(params,
+                             {'divid': expected_divid,
+                              'vid': vid,
+                              'rql': u'Any X WHERE X eid {}'.format(ou.eid)})
+
+
+class SEDAViewsTC(CubicWebTC):
 
     def test_seda_get_related_version(self):
         """Check that we get correct results when asking for `draft`, `published`, `replaced`
         version of a profile."""
         with self.admin_access.web_request() as req:
-            profile1 = testutils.publishable_profile(req, title=u'Profile 1')
+            profile1 = testutils.setup_profile(req, title=u'Profile 1')
             req.cnx.commit()
             profile1.cw_adapt_to('IWorkflowable').fire_transition('publish')
             req.cnx.commit()
             profile1.cw_clear_all_caches()
-            profile2 = testutils.publishable_profile(req, title=u'Profile 2', seda_replace=profile1)
+            profile2 = testutils.setup_profile(req, title=u'Profile 2', new_version_of=profile1)
             req.cnx.commit()
             profile2.cw_adapt_to('IWorkflowable').fire_transition('publish')
             req.cnx.commit()
             profile2.cw_clear_all_caches()
-            profile3 = testutils.publishable_profile(req, title=u'Profile 3', seda_replace=profile2)
+            profile3 = testutils.setup_profile(req, title=u'Profile 3', new_version_of=profile2)
             req.cnx.commit()
             profile3.cw_adapt_to('IWorkflowable').fire_transition('publish')
             req.cnx.commit()
             profile3.cw_clear_all_caches()
-            profile4 = testutils.publishable_profile(req, title=u'Profile 4', seda_replace=profile3)
+            profile4 = testutils.setup_profile(req, title=u'Profile 4', new_version_of=profile3)
             req.cnx.commit()
 
             def unwrap_generator(gen):
@@ -448,121 +294,6 @@ class SEDAFormsTC(CubicWebTC):
             self.assertEqual(unwrap_generator(box1.current_version(state=u'draft')).eid,
                              profile4.eid)
 
-    def test_seda_actions_notype_to_import(self):
-        from cubes.saem_ref.views import seda
-
-        with self.admin_access.web_request() as req:
-            profile = testutils.setup_seda_profile(req)
-            req.cnx.commit()
-            profile._cw = req  # XXX
-            actions = self.pactionsdict(req, profile.as_rset())
-            self.assertIn(seda.SEDAImportProfileArchiveObjectAction, actions['moreactions'])
-            self.assertNotIn(seda.SEDAImportProfileDocumentAction, actions['moreactions'])
-            actions = self.pactionsdict(req, profile.archives[0].as_rset())
-            self.assertIn(seda.SEDAImportProfileArchiveObjectAction, actions['moreactions'])
-            self.assertIn(seda.SEDAImportProfileDocumentAction, actions['moreactions'])
-        with self.new_access('anon').web_request() as req:
-            profile = req.entity_from_eid(profile.eid)
-            actions = self.pactionsdict(req, profile.as_rset())
-            self.assertNotIn(seda.SEDAImportProfileArchiveObjectAction, actions['moreactions'])
-            self.assertNotIn(seda.SEDAImportProfileDocumentAction, actions['moreactions'])
-
-    def test_seda_actions_not_exportable(self):
-        from cubes.saem_ref.views import seda
-
-        with self.admin_access.web_request() as req:
-            profile = req.create_entity('SEDAProfile', ark_naa=testutils.naa(req.cnx))
-            req.cnx.commit()
-            profile._cw = req  # XXX
-            actions = self.pactionsdict(req, profile.as_rset())
-            self.assertNotIn(seda.SEDA1DownloadAction, actions['moreactions'])
-            self.assertNotIn(seda.SEDA02DownloadAction, actions['moreactions'])
-            req.create_entity('ProfileArchiveObject',
-                              seda_name=req.create_entity('SEDAName'),
-                              seda_parent=profile)
-            req.cnx.commit()
-            profile.cw_clear_all_caches()
-        with self.admin_access.web_request() as req:
-            profile = req.entity_from_eid(profile.eid)
-            actions = self.pactionsdict(req, profile.as_rset())
-            self.assertIn(seda.SEDA1DownloadAction, actions['moreactions'])
-            self.assertIn(seda.SEDA02DownloadAction, actions['moreactions'])
-
-
-class SEDAComponentsImportTC(CubicWebTC):
-    """Tests for 'saem_ref.seda.doimport' controller (called from JavaScript)."""
-
-    def setup_database(self):
-        with self.admin_access.cnx() as cnx:
-            scheme = testutils.seda_scheme(
-                cnx, u'seda_document_type_code', u'preferred', u'QWE')
-            self.doctypecodevalue_eid = scheme.reverse_in_scheme[0].eid
-            doctypecode = self.doctypecode(cnx)
-            self.doc_eid = cnx.create_entity(
-                'ProfileDocument', user_cardinality=u'1',
-                user_annotation=u'plop',
-                seda_document_type_code=doctypecode).eid
-            profile = cnx.create_entity('SEDAProfile', ark_naa=testutils.naa(cnx))
-            self.archobj_eid = cnx.create_entity(
-                'ProfileArchiveObject',
-                seda_name=cnx.create_entity('SEDAName'),
-                seda_parent=profile).eid
-            cnx.commit()
-
-    def doctypecode(self, cnx):
-        return cnx.create_entity(
-            'SEDADocumentTypeCode',
-            seda_document_type_code_value=self.doctypecodevalue_eid)
-
-    def test_import_one_entity(self):
-        params = dict(eid=unicode(self.archobj_eid),
-                      cloned=unicode(self.doc_eid))
-        with self.admin_access.web_request(**params) as req:
-            path, _ = self.expect_redirect_handle_request(
-                req, 'saem_ref.seda.doimport')
-            etype, eid = path.split('/')
-            self.assertEqual(etype, 'ProfileArchiveObject'.lower())
-            clone = req.execute('Any X WHERE X seda_parent P, P eid %(p)s',
-                                {'p': eid}).one()
-            self.assertEqual([x.eid for x in clone.seda_clone_of], [self.doc_eid])
-            # Check that relationship targets have been copied.
-            self.assertEqual(
-                clone.seda_document_type_code[0].seda_document_type_code_value[0].eid,
-                self.doctypecodevalue_eid)
-            # Check that original entity attributes have been copied.
-            self.assertEqual(clone.user_cardinality, u'1')
-            self.assertEqual(clone.user_annotation, u'plop')
-
-    def test_import_multiple_entities(self):
-        with self.admin_access.cnx() as cnx:
-            doc_eid2 = cnx.create_entity(
-                'ProfileDocument', user_cardinality=u'0..1',
-                user_annotation=u'plip',
-                seda_document_type_code=self.doctypecode(cnx)).eid
-            doc_eid3 = cnx.create_entity(
-                'ProfileDocument', user_cardinality=u'1..n',
-                user_annotation=u'plouf',
-                seda_document_type_code=self.doctypecode(cnx)).eid
-            cnx.commit()
-        to_clone = [self.doc_eid, doc_eid2, doc_eid3]
-        params = dict(eid=unicode(self.archobj_eid),
-                      cloned=u','.join(map(unicode, to_clone)))
-        with self.admin_access.web_request(**params) as req:
-            path, _ = self.expect_redirect_handle_request(
-                req, 'saem_ref.seda.doimport')
-            etype, eid = path.split('/')
-            self.assertEqual(etype, 'ProfileArchiveObject'.lower())
-            rset = req.execute(
-                'Any X,O WHERE X seda_parent P, P eid %(p)s, X seda_clone_of O',
-                {'p': eid})
-            self.assertEqual(len(rset), 3)
-            self.assertCountEqual([oeid for __, oeid in rset.rows], to_clone)
-            cardinalities, annotations = zip(*[
-                (clone.user_cardinality, clone.user_annotation)
-                for clone in rset.entities()])
-            self.assertCountEqual(cardinalities, (u'1', u'0..1', u'1..n'))
-            self.assertCountEqual(annotations, (u'plop', u'plip', u'plouf'))
-
 
 class ArkViewsTC(CubicWebTC):
 
@@ -576,13 +307,20 @@ class ArkViewsTC(CubicWebTC):
             record = self.vreg['etypes'].etype_class('AuthorityRecord')(req)
             record.eid = 'A'
             fields = {
-                role_name('name', 'subject'): u'007',
                 role_name('ark', 'subject'): u'',
                 role_name('agent_kind', 'subject'): str(akind.eid),
                 role_name('ark_naa', 'subject'): str(naa_eid),
             }
-            req.form = self.fake_form('edition', entity_field_dicts=[(record, fields)])
-            eid = int(self.expect_redirect_handle_request(req)[0])
+            name = self.vreg['etypes'].etype_class('NameEntry')(req)
+            name.eid = 'B'
+            name_fields = {
+                role_name('parts', 'subject'): u'007',
+                role_name('name_entry_for', 'subject'): u'A',
+            }
+            req.form = self.fake_form('edition', entity_field_dicts=[(record, fields),
+                                                                     (name, name_fields)])
+            path = self.expect_redirect_handle_request(req)[0]
+            eid = eid_from_ark(path)
             record = req.cnx.entity_from_eid(eid)
             self.assertEqual(record.ark, u'0/r%09d' % eid)
 
@@ -595,7 +333,8 @@ class ArkViewsTC(CubicWebTC):
                       role_name('ark', 'subject'): u'',
                       role_name('authority', 'subject'): str(org.eid)}
             req.form = self.fake_form('edition', entity_field_dicts=[(agent, fields)])
-            eid = int(self.expect_redirect_handle_request(req)[0])
+            path = self.expect_redirect_handle_request(req)[0]
+            eid = eid_from_ark(path)
             agent = req.cnx.entity_from_eid(eid)
             self.assertEqual(agent.ark, u'0/oa%09d' % eid)
 
@@ -604,9 +343,10 @@ class ArkViewsTC(CubicWebTC):
             scheme = self.vreg['etypes'].etype_class('ConceptScheme')(req)
             scheme.eid = 'A'
             fields = {role_name('ark', 'subject'): u'',
-                      role_name('ark_naa', 'subject'): unicode(testutils.naa(req.cnx).eid)}
+                      role_name('ark_naa', 'subject'): text_type(testutils.naa(req.cnx).eid)}
             req.form = self.fake_form('edition', entity_field_dicts=[(scheme, fields)])
-            eid = int(self.expect_redirect_handle_request(req)[0])
+            path = self.expect_redirect_handle_request(req)[0]
+            eid = eid_from_ark(path)
             scheme = req.cnx.entity_from_eid(eid)
             self.assertEqual(scheme.ark, u'0/v%09d' % eid)
 
@@ -624,7 +364,8 @@ class ArkViewsTC(CubicWebTC):
                             role_name('label_of', 'subject'): 'A'}
             req.form = self.fake_form('edition', entity_field_dicts=[(concept, concept_fields),
                                                                      (label, label_fields)])
-            eid = int(self.expect_redirect_handle_request(req)[0])
+            path = self.expect_redirect_handle_request(req)[0]
+            eid = eid_from_ark(path)
             concept = req.cnx.entity_from_eid(eid)
             self.assertEqual(concept.ark, u'0/c%09d' % eid)
 
@@ -642,7 +383,8 @@ class ArkViewsTC(CubicWebTC):
             req.form = self.fake_form('edition', entity_field_dicts=[(concept, concept_fields),
                                                                      (label, label_fields)])
             req.form['__linkto'] = 'in_scheme:%s:subject' % scheme.eid
-            eid = int(self.expect_redirect_handle_request(req)[0])
+            path = self.expect_redirect_handle_request(req)[0]
+            eid = eid_from_ark(path)
             concept = req.cnx.entity_from_eid(eid)
             self.assertEqual(concept.ark, u'0/c%09d' % eid)
 
@@ -652,22 +394,6 @@ class ArkViewsTC(CubicWebTC):
             _pmid, rset = rewriter.rewrite(req, u'/ark:/JoE/Dalton')
             self.assertEqual(rset.printable_rql(), 'Any X WHERE X ark "JoE/Dalton"')
 
-    def test_download_filename(self):
-        with self.admin_access.web_request() as req:
-            cnx = req.cnx
-            record = testutils.authority_record(cnx, u'jim')
-            for ark, expected_filename in (
-                (u"", "EAC_{0}.xml".format(record.eid)),
-                (u"ZZZ/4242", "EAC_ZZZ_4242.xml"),
-            ):
-                record.cw_set(ark=ark)
-                view = self.vreg['views'].select('saem_ref.eac', req, record.as_rset())
-                view.set_request_content_type()
-                self.assertEqual(
-                    view._cw.headers_out.getRawHeaders('content-disposition'),
-                    ['attachment;filename="{0}"'.format(expected_filename)],
-                )
-
 
 class AssignArkWebServiceTC(CubicWebTC):
 
@@ -688,7 +414,7 @@ class AssignArkWebServiceTC(CubicWebTC):
     def test_authenticated_organization_method_not_post(self):
         with self.admin_access.cnx() as cnx:
             org_eid = testutils.authority_with_naa(cnx).eid
-        params = {'organization': unicode(org_eid)}
+        params = {'organization': text_type(org_eid)}
         with self.admin_access.web_request(
                 headers={'Accept': 'application/json'}, **params) as req:
             result = self.app_handle_request(req, 'ark')
@@ -708,7 +434,7 @@ class AssignArkWebServiceTC(CubicWebTC):
         # Specified eid does not match an Organization entity.
         with self.admin_access.cnx() as cnx:
             user_eid = cnx.find('CWUser')[0][0]
-        bad_params = {'organization': unicode(user_eid)}
+        bad_params = {'organization': text_type(user_eid)}
         with self.admin_access.web_request(
                 headers={'Accept': 'application/json'},
                 method='POST', **bad_params) as req:
@@ -718,9 +444,10 @@ class AssignArkWebServiceTC(CubicWebTC):
 
     def test_authenticated_organization_has_no_naa_method_post(self):
         with self.admin_access.cnx() as cnx:
-            org_eid = cnx.create_entity('Organization', name=u'dummy').eid
+            org_eid = cnx.create_entity(
+                'Organization', name=u'dummy', ark=u'123').eid
             cnx.commit()
-        bad_params = {'organization': unicode(org_eid)}
+        bad_params = {'organization': text_type(org_eid)}
         with self.admin_access.web_request(
                 headers={'Accept': 'application/json'},
                 method='POST', **bad_params) as req:
@@ -733,7 +460,7 @@ class AssignArkWebServiceTC(CubicWebTC):
         with self.admin_access.cnx() as cnx:
             org_eid = testutils.authority_with_naa(cnx).eid
             cnx.commit()
-        params = {'organization': unicode(org_eid)}
+        params = {'organization': text_type(org_eid)}
         with self.admin_access.web_request(
                 headers={'Accept': 'application/json'},
                 method='POST', **params) as req:
@@ -742,5 +469,41 @@ class AssignArkWebServiceTC(CubicWebTC):
                              [{'ark': '0/a000000001'}])
 
 
+class TimelineViewsTC(CubicWebTC):
+
+    def test_timeline_json(self):
+        with self.admin_access.cnx() as cnx:
+            ar1 = testutils.authority_record(
+                cnx, u'1',
+                start_date=date(1987, 6, 1), end_date=date(1988, 4, 2))
+            ar2 = testutils.authority_record(
+                cnx, u'2',
+                start_date=date(1989, 1, 1), end_date=date(1990, 4, 3))
+            ar3 = testutils.authority_record(
+                cnx, u'2',
+                start_date=date(1991, 1, 1), end_date=date(1992, 4, 3))
+            cnx.create_entity('ChronologicalRelation',
+                              description=u'avant avant',
+                              chronological_predecessor=ar1,
+                              chronological_successor=ar3)
+            cnx.create_entity('ChronologicalRelation',
+                              description=u'juste avant',
+                              chronological_predecessor=ar2,
+                              chronological_successor=ar3)
+            cnx.commit()
+        with self.admin_access.web_request() as req:
+            ar3 = req.entity_from_eid(ar3.eid)
+            ar2 = req.entity_from_eid(ar2.eid)
+            ar1 = req.entity_from_eid(ar1.eid)
+            data = json.loads(ar3.view(
+                'saem.authorityrecord-timeline-json', w=None))
+            predecessors = [x for x in data['timeline']['date']
+                            if x['tag'] == u'chronological_predecessor']
+            self.assertEqual(len(predecessors), 2)
+            self.assertCountEqual([p['headline'] for p in predecessors],
+                                  [ar2.view('incontext', w=None),
+                                   ar1.view('incontext', w=None)])
+
+
 if __name__ == '__main__':
     unittest.main()
