@@ -1,4 +1,4 @@
-# copyright 2015-2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -23,7 +23,7 @@ import pytz
 
 from cubicweb.devtools.testlib import CubicWebTC
 
-from cubes.saem_ref.hooks import extract_ark
+from cubicweb_saem_ref.hooks import extract_ark
 
 import testutils
 
@@ -90,47 +90,49 @@ class SAEMRefHooksTC(CubicWebTC):
 
     def test_sync_profile_md(self):
         with self.admin_access.repo_cnx() as cnx:
-            testutils.seda_scheme(cnx, u'seda_access_restriction_code', u'preferred',
-                                  u'AR038')
-            profile = testutils.setup_seda_profile(cnx)
-            profile.cw_set(modification_date=YESTERDAY)
+            scheme = testutils.scheme_for_type(cnx, 'seda_rule', 'SEDASeqAccessRuleRule', u'AR038')
+            transfer = testutils.setup_profile(cnx)
+            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(transfer)
             cnx.commit()
-            archive = profile.archives[0]
+            transfer.cw_set(modification_date=YESTERDAY)
+            cnx.commit()
+
             # edit composite children
-            archive.seda_name[0].cw_set(value=u'archive name')
+            unit_alt_seq.reverse_seda_title[0].cw_set(title=u'archive name')
             cnx.commit()
-            self.assertMDNow(profile)
-            self.resetMD(cnx, profile)
+            self.assertMDNow(transfer)
+            self.resetMD(cnx, transfer)
+            # edit relation to a composite children
+            code = scheme.reverse_in_scheme[0]
+            rule_seq = cnx.create_entity('SEDASeqAccessRuleRule',
+                                         reverse_seda_start_date=cnx.create_entity('SEDAStartDate'),
+                                         seda_rule=code)
+            rule = cnx.create_entity('SEDAAccessRule',
+                                     seda_access_rule=transfer,
+                                     seda_seq_access_rule_rule=rule_seq)
+            cnx.commit()
+            self.assertMDNow(transfer)
+            self.resetMD(cnx, transfer)
+            # edit link from composite children to an entity which is not part of the container
+            rule_seq.cw_set(seda_rule=None)
+            cnx.commit()
+            self.assertMDNow(transfer)
+            self.resetMD(cnx, transfer)
+            # deletion of a composite children
+            rule.cw_delete()
+            cnx.commit()
+            self.assertMDNow(transfer)
+            transfer.cw_adapt_to('IWorkflowable').fire_transition('publish')
+            cnx.commit()
+            self.resetMD(cnx, transfer)
             # edit link from root to an entity which is not part of the container
+            # XXX that shouldn't change anything for this special relation
             agent = testutils.organization_unit(
                 cnx, u'marcel', archival_roles=['deposit'])
-            profile.cw_set(seda_transferring_agent=agent)
+            transfer.cw_set(reverse_use_profile=agent)
             cnx.commit()
-            self.assertMDNow(profile)
-            self.resetMD(cnx, profile)
-            # edit link from composite children to an entity which is not part of the container
-            code = testutils.concept(cnx, 'AR038')
-            archive.seda_access_restriction_code[0].cw_set(seda_access_restriction_code_value=code)
-            cnx.commit()
-            self.assertMDNow(profile)
-            self.resetMD(cnx, profile)
-            archive.seda_access_restriction_code[0].cw_set(seda_access_restriction_code_value=None)
-            cnx.commit()
-            self.assertMDNow(profile)
-            self.resetMD(cnx, profile)
-            # edit relation to a composite children
-            arc = cnx.create_entity('SEDAAppraisalRuleCode')
-            ard = cnx.create_entity('SEDAAppraisalRuleDuration')
-            ar = cnx.create_entity('SEDAAppraisalRule', seda_appraisal_rule_code=arc,
-                                   seda_appraisal_rule_duration=ard)
-            archive.cw_set(seda_appraisal_rule=ar)
-            cnx.commit()
-            self.assertMDNow(profile)
-            self.resetMD(cnx, profile)
-            ar.cw_delete()
-            cnx.commit()
-            self.assertMDNow(profile)
-            self.resetMD(cnx, profile)
+            self.assertMDNow(transfer)
+            self.resetMD(cnx, transfer)
 
     def test_externaluri_to_concept(self):
         with self.admin_access.repo_cnx() as cnx:
@@ -205,19 +207,19 @@ class SAEMRefHooksTC(CubicWebTC):
 
     def test_ark_generation_seda_profile(self):
         with self.admin_access.repo_cnx() as cnx:
-            profile = testutils.setup_seda_profile(cnx)
+            profile = testutils.setup_profile(cnx)
             self.assertEqual(profile.ark, '0/p{0:09d}'.format(profile.eid))
             self.assertEqual(profile.cwuri, 'ark:/0/p{0:09d}'.format(profile.eid))
 
     def test_ark_generation_seda_profile_ark_given(self):
         with self.admin_access.repo_cnx() as cnx:
-            profile = testutils.setup_seda_profile(cnx, ark=u'authority/124')
+            profile = testutils.setup_profile(cnx, ark=u'authority/124')
             self.assertEqual(profile.ark, 'authority/124')
             self.assertEqual(profile.cwuri, 'ark:/authority/124')
 
     def test_ark_generation_seda_profile_with_cwuri(self):
         with self.admin_access.repo_cnx() as cnx:
-            profile = testutils.setup_seda_profile(
+            profile = testutils.setup_profile(
                 cnx, cwuri=u'http://example.org/profile/125')
             self.assertEqual(profile.ark, '0/p{0:09d}'.format(profile.eid))
             self.assertEqual(profile.cwuri, 'http://example.org/profile/125')
@@ -250,6 +252,45 @@ class SAEMRefHooksTC(CubicWebTC):
             self.assertEqual(concept.ark, '67717/1234')
             self.assertEqual(concept.cwuri, 'http://dcf/res/ark:/67717/1234')
 
+    def test_user_email(self):
+        # EmailAddress belong to the Organisation graph, hence may conflict with hooks when linked
+        # to a user. This is a non-regression test checking that no error is raised on attempt to
+        # add an email to a user (which is not adaptable to IContained nor IContainer)
+        with self.admin_access.repo_cnx() as cnx:
+            cnx.create_entity('EmailAddress', address=u'admin@cubicweb.org',
+                              reverse_use_email=cnx.user.eid)
+            cnx.commit()
+
+
+class AuthorityRecordHookTC(CubicWebTC):
+
+    def setup_database(self):
+        with self.admin_access.cnx() as cnx:
+            self.org_eid = testutils.authority_with_naa(cnx).eid
+            self.orgunit_eid = testutils.organization_unit(
+                cnx, u'test', archival_roles=('archival', ),
+                reverse_archival_unit=self.org_eid, authority=self.org_eid).eid
+            self.create_user(cnx, u'bob', authority=self.org_eid)
+            cnx.commit()
+
+    def test_automatic_use_authorityrecord(self):
+        with self.new_access(u'bob').cnx() as cnx:
+            arecord = testutils.authority_record(cnx, u'test')
+            cnx.commit()
+            self.assertEqual(
+                [ou.eid for ou in arecord.reverse_use_authorityrecord],
+                [self.orgunit_eid])
+
+    def test_automatic_use_authorityrecord_no_archival_unit(self):
+        with self.admin_access.cnx() as cnx:
+            self.create_user(cnx, u'alice')
+            cnx.commit()
+        with self.new_access(u'alice').cnx() as cnx:
+            arecord = testutils.authority_record(cnx, u'test')
+            cnx.commit()
+            arecord.cw_clear_all_caches()
+            self.assertFalse(arecord.reverse_use_authorityrecord)
+
 
 class ExtractArkTC(unittest.TestCase):
 
@@ -270,14 +311,13 @@ class EntityLifeCycleTC(CubicWebTC):
     def setup_database(self):
         with self.admin_access.repo_cnx() as cnx:
             bob = self.create_user(cnx, u'bob',
-                                   authority=cnx.find('Organization').one())
-            bob_agent = testutils.agent(cnx, u'bob', agent_user=bob)
+                                   authority=testutils.authority_with_naa(cnx))
             cnx.commit()
-            self.bob_eid = bob_agent.eid
+            self.bob_eid = bob.eid
 
     def _check_create(self, cnx, eid, msg):
         activity = cnx.find('Activity', type=u'create', used=eid).one()
-        self.assertEqual(activity.associated_with[0].eid, self.bob_eid)
+        self.assertEqual(activity.associated_with[0].eid, cnx.user.eid)
         self.assertEqual(activity.generated[0].eid, eid)
         self.assertEqual(activity.description, msg)
 
@@ -294,10 +334,10 @@ class EntityLifeCycleTC(CubicWebTC):
             cnx.commit()
             self._check_create(cnx, authorityrecord.eid,
                                'created authorityrecord')
-            authorityrecord.cw_set(isni=u'123', name=u'Adam')
+            authorityrecord.cw_set(isni=u'123', record_id=u'Adam')
             cnx.commit()
             self._check_modification(cnx, authorityrecord.eid,
-                                     'modified isni, name')
+                                     'modified isni, record_id')
 
     def test_authorityrecord_add_component(self):
         with self.new_access(u'bob').repo_cnx() as cnx:
@@ -384,11 +424,11 @@ class EntityLifeCycleTC(CubicWebTC):
             f1.cw_delete()
             cnx.create_entity('AgentFunction', name=u'mib',
                               function_agent=authorityrecord)
-            authorityrecord.cw_set(isni=u'123', name=u'Adam')
+            authorityrecord.cw_set(isni=u'123', record_id=u'Adam')
             cnx.commit()
             self._check_modification(
                 cnx, authorityrecord.eid,
-                '* modified function_agent_object, general_context_of_object, isni, name\n'
+                '* modified function_agent_object, general_context_of_object, isni, record_id\n'
                 '* added place_address, place_agent_object')
 
     def test_authorityrecord2authorityrecord_relation(self):
@@ -403,8 +443,7 @@ class EntityLifeCycleTC(CubicWebTC):
 
     def test_no_activity_generated(self):
         with self.admin_access.repo_cnx() as cnx:
-            agent = cnx.create_entity('Agent', name=u'smith',
-                                      authority=cnx.find('Organization')[0][0])
+            agent = testutils.agent(cnx, u'smith', authority=cnx.find('Organization')[0][0])
             cnx.commit()
             authorityrecord = testutils.authority_record(
                 cnx, u'Smith', reverse_authority_record=agent)
@@ -415,12 +454,16 @@ class EntityLifeCycleTC(CubicWebTC):
                                  used=authorityrecord.eid))
 
     def test_concept(self):
-        with self.new_access(u'bob').repo_cnx() as cnx:
+        with self.admin_access.repo_cnx() as cnx:
             scheme = cnx.create_entity('ConceptScheme', title=u'my thesaurus',
                                        ark_naa=testutils.naa(cnx))
             cnx.commit()
             # concept scheme creation
             self._check_create(cnx, scheme.eid, 'created conceptscheme')
+
+        with self.new_access(u'bob').repo_cnx() as cnx:
+            scheme = cnx.entity_from_eid(scheme.eid)
+
             concept = scheme.add_concept(u'hello')
             cnx.commit()
             # concept scheme add concept
@@ -439,32 +482,40 @@ class EntityLifeCycleTC(CubicWebTC):
             self.assertEqual(len(rset), 2)
             self.assertEqual(rset.get_entity(0, 0).description, 'added in_scheme_object')
 
-
-class SEDAProfileHooksTC(CubicWebTC):
-
-    def test_archive_created(self):
-        """Test hook creating a Profile Archive upon creation of a SEDAProfile."""
-        with self.admin_access.repo_cnx() as cnx:
-            profile = testutils.setup_seda_profile(cnx, complete_archive=False)
+    def test_archive_transfer_create_update(self):
+        with self.new_access(u'bob').cnx() as cnx:
+            transfer = testutils.setup_profile(cnx)
             cnx.commit()
-            arch = profile.archives[0]
-            self.assertTrue(arch.seda_access_restriction_code)
-            self.assertTrue(arch.seda_content_description)
-            content_desc = arch.content_description
-            self.assertTrue(content_desc.seda_description_level)
-            self.assertEqual(profile.support_seda_exports, 'SEDA-0.2.xsd, SEDA-1.0.xsd')
+            self._check_create(cnx, transfer.eid, 'created sedaarchivetransfer')
+            transfer.cw_set(title=u'123')
+            cnx.commit()
+            self._check_modification(cnx, transfer.eid,
+                                     'modified title')
+
+    def test_nonregr_concept_container(self):
+        with self.admin_access.repo_cnx() as cnx:
+            scheme = cnx.create_entity('ConceptScheme', title=u'my thesaurus',
+                                       ark_naa=testutils.naa(cnx))
+            concept = scheme.add_concept(u'hello')
+            cnx.commit()
+            concept.cw_delete()
+            cnx.commit()
+
+
+class SEDAArchiveTransferHooksTC(CubicWebTC):
 
     def test_profile_deprecated(self):
         """Test hook deprecating a SEDA Profile upon successor publication."""
         with self.admin_access.repo_cnx() as cnx:
-            profile = testutils.setup_seda_profile(cnx, complete_archive=False)
+            profile = testutils.setup_profile(cnx)
             cnx.commit()
             workflow = profile.cw_adapt_to('IWorkflowable')
             workflow.fire_transition('publish')
             cnx.commit()
             profile.cw_clear_all_caches()
             self.assertEqual(workflow.state, 'published')
-            cloned = cnx.create_entity('SEDAProfile', title=u'Clone', seda_replace=profile,
+            cloned = cnx.create_entity('SEDAArchiveTransfer', title=u'Clone',
+                                       new_version_of=profile,
                                        ark_naa=testutils.naa(cnx))
             cnx.commit()
             cloned.cw_adapt_to('IWorkflowable').fire_transition('publish')
@@ -472,22 +523,6 @@ class SEDAProfileHooksTC(CubicWebTC):
             profile.cw_clear_all_caches()
             self.assertEqual(workflow.state, 'deprecated')
 
-    def test_profile_export_compat(self):
-        with self.admin_access.client_cnx() as cnx:
-            profile = cnx.create_entity('SEDAProfile', ark_naa=testutils.naa(cnx))
-            cnx.commit()
-            self.assertEqual(profile.support_seda_exports, '')
-            ao = cnx.create_entity('ProfileArchiveObject',
-                                   user_cardinality=u'0..1',
-                                   seda_name=cnx.create_entity('SEDAName'),
-                                   seda_parent=profile)
-            cnx.commit()
-            self.assertEqual(profile.support_seda_exports, '')
-            ao.cw_set(user_cardinality=u'1')
-            cnx.commit()
-            profile.cw_clear_all_caches()
-            self.assertEqual(profile.support_seda_exports, 'SEDA-0.2.xsd, SEDA-1.0.xsd')
-
 
 if __name__ == '__main__':
     unittest.main()
